/**
 * 
 */
package org.gcube.portlets.user.td.csvexportwidget.client.progress;

import java.util.ArrayList;

import org.gcube.portlets.user.td.gwtservice.client.rpc.TDGWTServiceAsync;
import org.gcube.portlets.user.td.gwtservice.shared.csv.CSVExportMonitor;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.sencha.gxt.widget.core.client.box.AlertMessageBox;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;

/**
 * 
 * @author "Giancarlo Panichi" 
 * <a href="mailto:g.panichi@isti.cnr.it">g.panichi@isti.cnr.it</a> 
 *
 */

public class CSVExportProgressUpdater extends Timer {

	protected ArrayList<CSVExportProgressListener> listeners = new ArrayList<CSVExportProgressListener>();

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void run() {
		Log.debug("requesting operation progress");
		TDGWTServiceAsync.INSTANCE
				.getCSVExportMonitor(new AsyncCallback<CSVExportMonitor>() {

					
					public void onFailure(Throwable caught) {
						cancel();
						Log.error("Error retrieving the operation state",
								caught);
						String message = getStack(caught);
						fireOperationFailed(caught,
								"Failed getting operation updates", message);
					}

					public void onSuccess(CSVExportMonitor result) {
						Log.info("retrieved CSVImportMonitor: "
								+ result.getStatus());
						switch (result.getStatus()) {
						case INITIALIZING:
							Log.info("CSV Export Initializing...");
							fireOperationInitializing();
							break;
						case ABORTED:
							cancel();
							Log.info("CSV Export Operation Aborted");
							break;
						case IN_PROGRESS:
							fireOperationUpdate(result.getProgress());
							break;
						case FAILED:
							cancel();
							errorMessage(result);
							break;
						case SUCCEDED:
							cancel();
							Log.info("Export fisnish ExportMetadata:"
									+ result.getTrExportMetadata());
							fireOperationComplete();
							break;

						default:
							Log.info("Unknow State");
							break;
						}

					}

				});

	}

	protected void errorMessage(CSVExportMonitor result) {
		Log.info("CSV Export Failed");
		Throwable th = null;
		String failure = null;
		String details = null;
		if (result.getError() != null) {
			th = result.getError();
			failure = "Failed Client Library Export";
			details = result.getError().getLocalizedMessage();
		} else {
			th = new Throwable("Failed");
			failure = "Failed Client Library Export";
			details = "Error in export";
		}
		fireOperationFailed(th, failure, details);

		AlertMessageBox d = new AlertMessageBox("Error in CSV Export", details);
		d.addHideHandler(new HideHandler() {

			
			public void onHide(HideEvent event) {
				//
			}

		});
		d.show();

	}

	protected String getStack(Throwable e) {
		String message = e.getLocalizedMessage() + " -> <br>";
		Throwable c = e.getCause();
		if (c != null)
			message += getStack(c);
		return message;
	}

	protected void fireOperationInitializing() {
		for (CSVExportProgressListener listener : listeners)
			listener.operationInitializing();
	}

	protected void fireOperationUpdate(float elaborated) {
		for (CSVExportProgressListener listener : listeners)
			listener.operationUpdate(elaborated);
	}

	protected void fireOperationComplete() {
		for (CSVExportProgressListener listener : listeners)
			listener.operationComplete();
	}

	protected void fireOperationFailed(Throwable caught, String failure,
			String failureDetails) {
		for (CSVExportProgressListener listener : listeners)
			listener.operationFailed(caught, failure, failureDetails);
	}

	/**
	 * Add a new {@link CSVExportProgressListener} to this
	 * {@link CSVExportProgressUpdater}.
	 * 
	 * @param listener
	 *            the listener to add.
	 */
	public void addListener(CSVExportProgressListener listener) {
		listeners.add(listener);
	}

	/**
	 * Removes the specified {@link CSVExportProgressListener} from this
	 * {@link CSVExportProgressUpdater}.
	 * 
	 * @param listener
	 *            the listener to remove.
	 */
	public void removeListener(CSVExportProgressListener listener) {
		listeners.remove(listener);
	}
}
